import tkinter as tk
from typing import Any

import ttkbootstrap as ttk


class Connector(ttk.Frame):  # type: ignore[misc]
    def __init__(self, container: ttk.Frame) -> None:
        super().__init__(container)

        self._init_widgets()

    def _init_widgets(self) -> None:
        self._host_ip = tk.StringVar(self)
        self._host_ip.trace_add("write", self._on_input)

        self._ip_input = ttk.Entry(self, textvariable=self._host_ip)
        self._ip_input.bind("<Return>", self._on_pair)
        self._ip_input.pack(fill="x", pady=(0, 5))

        self._pair_button = ttk.Button(self, text="pair", command=self._on_pair)
        self._pair_button.pack(fill="x")

        self.pack()

    def _on_input(self, *_: Any) -> None:
        self.event_generate("<<Input>>")

    def _on_pair(self, *_: Any) -> None:
        if self.enabled:
            self.event_generate("<<Pair>>")

    @property
    def host_ip(self) -> str:
        return self._host_ip.get()

    @host_ip.setter
    def host_ip(self, value: str) -> None:
        self._host_ip.set(value)

    @property
    def enabled(self) -> bool:
        return bool(str(self._pair_button["state"]) == "normal")

    @enabled.setter
    def enabled(self, value: bool) -> None:
        self._pair_button["state"] = "normal" if value else "disabled"

    def focus(self) -> None:
        self._ip_input.focus()



import tkinter as tk
from tkinter import font

from ttkbootstrap.window import Window

from .frame import AppFrame
from .storage import AppData


class App(Window):  # type: ignore
    def __init__(self) -> None:
        super().__init__(title="PhilipsTV GUI")

        font.nametofont("TkDefaultFont").config(size=11)
        font.nametofont("TkTextFont").config(size=11)

        self._app_data = AppData.load()

        self._init_widgets()

    def _init_widgets(self) -> None:
        self.columnconfigure(0, weight=1)
        self.rowconfigure(0, weight=1)

        AppFrame(self, self._app_data).grid(row=0, column=0, sticky=tk.NSEW)

        self.grid()



import tkinter as tk
from enum import Enum
from functools import partial
from tkinter import Button as LegacyButton
from typing import Any, Dict, Optional, Tuple

import ttkbootstrap as ttk
from philipstv import AmbilightColor, PhilipsTVRemote
from ttkbootstrap.colorutils import color_to_hex, contrast_color
from ttkbootstrap.dialogs.colorchooser import ColorChooserDialog
from ttkbootstrap.tooltip import ToolTip

from philipstv_gui.errors import handle_remote_errors


class AmbilightSide(Enum):
    LEFT = "left"
    TOP = "top"
    RIGHT = "right"
    BOTTOM = "bottom"


Color = Tuple[int, int, int]


DEFAULT_COLOR = (255, 255, 255)


class ColorButton(LegacyButton):
    def __init__(self, container: ttk.Frame, **kwargs: Any) -> None:
        super().__init__(  # type: ignore[call-arg]
            container, autostyle=False, relief=tk.FLAT, **kwargs
        )
        self.set_color(DEFAULT_COLOR)

    def set_color(self, color: Color) -> None:
        hex_color = color_to_hex(color)
        contrast_hex_color = contrast_color(color)
        self.configure(
            background=hex_color,
            activebackground=hex_color,
            foreground=contrast_hex_color,
            activeforeground=contrast_hex_color,
        )


class Ambilight(ttk.Frame):  # type: ignore[misc]
    def __init__(self, container: ttk.Frame, remote: Optional[PhilipsTVRemote]) -> None:
        super().__init__(container)

        self.remote = remote

        self._colors: Dict[AmbilightSide, Color] = {side: DEFAULT_COLOR for side in AmbilightSide}
        self._color_buttons: Dict[AmbilightSide, ColorButton] = {}

        self._init_widgets()
        self.refresh()

    def _init_widgets(self) -> None:
        self.columnconfigure(0, weight=1)
        self.rowconfigure(1, weight=1)

        settings_frame = ttk.Labelframe(self, text="Settings")
        settings_frame.grid(row=0, column=0, sticky=tk.EW, padx=2, pady=2)

        self._power_toggle = ttk.Checkbutton(
            settings_frame,
            text="Ambilight enabled",
            bootstyle="round-toggle",
            command=self._toggle_power,
        )
        self._power_toggle.pack(fill=tk.BOTH, padx=2, pady=2)

        self._lock_toggle = ttk.Checkbutton(
            settings_frame, text="Same color on all sides", bootstyle="round-toggle"
        )
        self._lock_toggle.pack(fill=tk.BOTH, padx=2, pady=2)

        colors_frame = ttk.Labelframe(self, text="Colors")
        colors_frame.grid(row=1, column=0, sticky=tk.NSEW, padx=2, pady=2)

        for side in AmbilightSide:
            button = ColorButton(
                colors_frame,
                command=partial(self._select_color, side),
                text=side.value.capitalize(),
            )
            button.pack(fill=tk.BOTH, padx=2, pady=2)
            ToolTip(button, text="Click to adjust")
            self._color_buttons[side] = button

        self.grid()

    def refresh(self) -> None:
        if not (self.remote and self.remote.auth):
            return
        with handle_remote_errors(self):
            self._ambilight_enabled = self.remote.get_ambilight_power()

    def _toggle_power(self) -> None:
        if not (self.remote and self.remote.auth):
            return
        with handle_remote_errors(self):
            self.remote.set_ambilight_power(self._ambilight_enabled)
        if self._ambilight_enabled:
            self._apply_colors()

    def _select_color(self, selected_side: AmbilightSide) -> None:
        color_chooser = ColorChooserDialog()
        color_chooser.show()
        color = color_chooser.result
        if not color:
            return
        sides_to_set = {selected_side} if not self._sides_locked else set(AmbilightSide)
        for side in sides_to_set:
            self._set_side_color(side, color.rgb)
        self._apply_colors()

    def _set_side_color(self, side: AmbilightSide, color: Color) -> None:
        self._colors[side] = color
        self._color_buttons[side].set_color(color)

    def _apply_colors(self) -> None:
        if not (self.remote and self.remote.auth):
            return
        if not self._ambilight_enabled:
            return
        ambilight_values = {
            side.value: AmbilightColor.from_tuple(color) for side, color in self._colors.items()
        }
        with handle_remote_errors(self):
            self.remote.set_ambilight_color(**ambilight_values)

    @property
    def _ambilight_enabled(self) -> bool:
        return "selected" in self._power_toggle.state()

    @_ambilight_enabled.setter
    def _ambilight_enabled(self, state: bool) -> None:
        self._power_toggle.state(["selected"] if state else [])

    @property
    def _sides_locked(self) -> bool:
        return "selected" in self._lock_toggle.state()

    @_sides_locked.setter
    def _sides_locked(self, state: bool) -> None:
        self._lock_toggle.state(["selected"] if state else [])



import tkinter as tk
from typing import Any, Optional

import ttkbootstrap as ttk
from philipstv import PhilipsTVRemote
from philipstv.exceptions import PhilipsTVError, PhilipsTVPairingError
from ttkbootstrap.dialogs.dialogs import Messagebox, Querybox

from philipstv_gui.ambilight import Ambilight
from philipstv_gui.applications import Applications
from philipstv_gui.channels import Channels
from philipstv_gui.connector import Connector

from .errors import connection_error, pairing_error
from .remote import Remote
from .storage import AppData, HostData


class AbortPairing(Exception):
    pass


class AppFrame(ttk.Frame):  # type: ignore[misc]
    def __init__(self, container: tk.Tk, store: AppData) -> None:
        super().__init__(container)

        self._store: AppData = store
        self._remote: Optional[PhilipsTVRemote] = None

        self._init_widgets()

        if last_host := self._store.last_host:
            self._remote = PhilipsTVRemote.new(last_host.host, (last_host.id, last_host.key))

        self.refresh()

    def _init_widgets(self) -> None:
        self.columnconfigure(0, weight=1)
        self.rowconfigure(0, weight=0)
        self.rowconfigure(1, weight=1)

        self._connector_panel = Connector(self)
        self._connector_panel.grid(row=0, column=0, sticky=tk.EW, padx=5, pady=5)
        self._connector_panel.bind("<<Pair>>", self._on_pair)
        self._connector_panel.bind("<<Input>>", self._on_input)

        notebook = ttk.Notebook(self)
        notebook.grid(row=1, column=0, sticky=tk.NSEW, padx=5, pady=5)

        self._remote_panel = Remote(notebook, self._remote)
        notebook.add(self._remote_panel, text="Remote")

        self._channels_panel = Channels(notebook, self._remote)
        notebook.add(self._channels_panel, text="Channels")

        self._apps_panel = Applications(notebook, self._remote)
        notebook.add(self._apps_panel, text="Apps")

        self._ambilight_panel = Ambilight(self, self._remote)
        notebook.add(self._ambilight_panel, text="Ambilight")

        self.grid()

    def refresh(self) -> None:
        self._remote_panel.remote = self._remote
        self._channels_panel.remote = self._remote
        self._apps_panel.remote = self._remote
        self._ambilight_panel.remote = self._remote

        if self._remote:
            self._connector_panel.host_ip = self._remote.host
            self._connector_panel.enabled = not bool(self._remote.auth)
        else:
            self._connector_panel.focus()

        self._channels_panel.refresh()
        self._apps_panel.refresh()
        self._ambilight_panel.refresh()

    def _on_pair(self, _: Any) -> None:
        if not self._connector_panel.host_ip:
            Messagebox.show_error("Enter the IP address!", "Pairing error", parent=self)
            return

        remote = PhilipsTVRemote.new(self._connector_panel.host_ip)

        try:
            credentials = remote.pair(self._ask_for_pin)
        except AbortPairing:
            pass
        except PhilipsTVPairingError as exc:
            pairing_error(self, exc)
        except PhilipsTVError:
            connection_error(self)
        else:
            self._remote = remote
            self._store.last_host = HostData(remote.host, *credentials)
            self._store.save()
        finally:
            self.refresh()

    def _on_input(self, _: Any) -> None:
        input_same_as_current = self._remote and self._remote.host == self._connector_panel.host_ip
        self._connector_panel.enabled = not input_same_as_current

    def _ask_for_pin(self) -> str:
        response = Querybox.get_string("Enther PIN number displayed on the TV", "PIN", parent=self)
        if response is None:
            raise AbortPairing
        return response  # type: ignore[no-any-return]



import json
from dataclasses import asdict, dataclass
from pathlib import Path
from typing import Any, Dict, Optional

from appdirs import user_data_dir

DATA_FILE = Path(user_data_dir("philipstv-gui", "cyran.dev")) / "data.json"


@dataclass(frozen=True)
class HostData:
    host: str
    id: str
    key: str

    @classmethod
    def from_dict(cls, raw_dict: Dict[str, Any]) -> "HostData":
        return cls(host=raw_dict["host"], id=raw_dict["id"], key=raw_dict["key"])


@dataclass
class AppData:
    last_host: Optional[HostData]

    @classmethod
    def load(cls) -> "AppData":
        raw_data = {}
        if DATA_FILE.exists():
            raw_data = json.loads(DATA_FILE.read_text())
        last_host = HostData.from_dict(raw_data["last_host"]) if raw_data.get("last_host") else None
        return cls(last_host=last_host)

    def save(self) -> None:
        DATA_FILE.parent.mkdir(parents=True, exist_ok=True)
        DATA_FILE.touch(exist_ok=True)
        DATA_FILE.write_text(json.dumps(asdict(self)))



from contextlib import contextmanager
from typing import Iterator

import ttkbootstrap as ttk
from philipstv.exceptions import PhilipsTVAPIUnauthorizedError, PhilipsTVError
from ttkbootstrap.dialogs.dialogs import Messagebox


def pairing_error(parent: ttk.Frame, exception: Exception) -> None:
    Messagebox.show_error(str(exception), "Pairing error", parent=parent)


def not_paired_error(parent: ttk.Frame) -> None:
    authentication_error(parent)


def authentication_error(parent: ttk.Frame) -> None:
    Messagebox.show_error("First pair with the TV!", "Not authenticated", parent=parent)


def connection_error(parent: ttk.Frame) -> None:
    Messagebox.show_error(
        "Could not connect with the TV. "
        "Make sure the IP address is correct and TV is powered on.",
        "TV connection error",
        parent=parent,
    )


@contextmanager
def handle_remote_errors(parent: ttk.Frame) -> Iterator[None]:
    try:
        yield
    except PhilipsTVAPIUnauthorizedError:
        authentication_error(parent)
    except PhilipsTVError:
        connection_error(parent)



import tkinter as tk
from dataclasses import dataclass, field
from functools import partial
from typing import Any, Callable, List, Optional

import ttkbootstrap as ttk
from philipstv import PhilipsTVRemote
from philipstv.model import InputKeyValue
from ttkbootstrap.dialogs import Dialog

from .errors import handle_remote_errors, not_paired_error

ButtonCallback = Callable[[InputKeyValue], None]


@dataclass(frozen=True)
class RemoteButton:
    row: int
    column: int
    label: str
    value: InputKeyValue
    keys: List[str] = field(default_factory=list)
    rowspan: int = 1
    colspan: int = 1

    def install(
        self,
        container: ttk.Frame,
        click_callback: ButtonCallback,
        key_callback: ButtonCallback,
    ) -> ttk.Button:
        button = ttk.Button(
            container,
            text=self.label,
            command=partial(click_callback, self.value),
            bootstyle="dark-outline",
        )
        button.grid(
            row=self.row,
            column=self.column,
            rowspan=self.rowspan,
            columnspan=self.colspan,
            sticky=tk.NSEW,
            padx=1,
            pady=1,
        )
        for key in self.keys:
            container.bind_all(f"<Key-{key}>", partial(key_callback, self.value))
        return button


BUTTONS = [
    RemoteButton(0, 0, "power", InputKeyValue.STANDBY, colspan=3),
    RemoteButton(1, 0, "ambilight", InputKeyValue.AMBILIGHT_ON_OFF, colspan=3),
    RemoteButton(2, 0, "adjust", InputKeyValue.ADJUST),
    RemoteButton(2, 1, "tv", InputKeyValue.WATCH_TV, ["t"]),
    RemoteButton(2, 2, "source", InputKeyValue.SOURCE, ["i"]),
    RemoteButton(3, 1, "up", InputKeyValue.CURSOR_UP, ["Up", "w"]),
    RemoteButton(4, 0, "left", InputKeyValue.CURSOR_LEFT, ["Left", "a"]),
    RemoteButton(4, 1, "ok", InputKeyValue.CONFIRM, ["Return"]),
    RemoteButton(4, 2, "right", InputKeyValue.CURSOR_RIGHT, ["Right", "d"]),
    RemoteButton(5, 1, "down", InputKeyValue.CURSOR_DOWN, ["Down", "s"]),
    RemoteButton(6, 0, "back", InputKeyValue.BACK, ["BackSpace", "Escape"]),
    RemoteButton(6, 1, "home", InputKeyValue.HOME, ["Home", "h"]),
    RemoteButton(6, 2, "options", InputKeyValue.OPTIONS, ["o"]),
    RemoteButton(7, 0, "rewind", InputKeyValue.REWIND, ["bracketleft"]),
    RemoteButton(7, 1, "play", InputKeyValue.PLAY_PAUSE, ["space"]),
    RemoteButton(7, 2, "forward", InputKeyValue.FAST_FORWARD, ["bracketright"]),
    RemoteButton(8, 0, "stop", InputKeyValue.STOP),
    RemoteButton(8, 1, "pause", InputKeyValue.PAUSE, ["p"]),
    RemoteButton(8, 2, "record", InputKeyValue.RECORD, ["r"]),
    RemoteButton(9, 0, "vol+", InputKeyValue.VOLUME_UP, ["plus"]),
    RemoteButton(9, 1, "info", InputKeyValue.INFO, ["question"]),
    RemoteButton(9, 2, "ch+", InputKeyValue.CHANNEL_STEP_UP, ["Prior"]),
    RemoteButton(10, 0, "vol-", InputKeyValue.VOLUME_DOWN, ["minus"]),
    RemoteButton(10, 1, "mute", InputKeyValue.MUTE, ["m"]),
    RemoteButton(10, 2, "ch-", InputKeyValue.CHANNEL_STEP_DOWN, ["Next"]),
    RemoteButton(11, 0, "1", InputKeyValue.DIGIT_1, ["1"]),
    RemoteButton(11, 1, "2", InputKeyValue.DIGIT_2, ["2"]),
    RemoteButton(11, 2, "3", InputKeyValue.DIGIT_3, ["3"]),
    RemoteButton(12, 0, "4", InputKeyValue.DIGIT_4, ["4"]),
    RemoteButton(12, 1, "5", InputKeyValue.DIGIT_5, ["5"]),
    RemoteButton(12, 2, "6", InputKeyValue.DIGIT_6, ["6"]),
    RemoteButton(13, 0, "7", InputKeyValue.DIGIT_7, ["7"]),
    RemoteButton(13, 1, "8", InputKeyValue.DIGIT_8, ["8"]),
    RemoteButton(13, 2, "9", InputKeyValue.DIGIT_9, ["9"]),
    RemoteButton(14, 0, "text", InputKeyValue.TELETEXT),
    RemoteButton(14, 1, "0", InputKeyValue.DIGIT_0, ["0"]),
    RemoteButton(14, 2, "subtitle", InputKeyValue.SUBTITLE),
]


class CapHelpDialog(Dialog):  # type: ignore[misc]
    _HELP_TEXT = """\
    Keyboard capture is a feature allowing you to interact with the TV by pressing keyboard keys. \
    Keyboard keys are mapped to the TV remote buttons. Please remember that the application window \
    has to be focused for this to work.

    Available mappings:"""

    _KEY_DISPLAY_MAP = {
        "bracketleft": "[",
        "bracketright": "]",
        "minus": "-",
        "plus": "+",
        "Prior": "Page Down",
        "Next": "Page Up",
        "question": "?",
    }

    def __init__(self, container: ttk.Frame) -> None:
        super().__init__(container, title="PhilipsTV GUI: Keyboard capture help")

    def create_body(self, container: ttk.Frame) -> None:
        frame = ttk.Frame(container)
        frame.pack()

        text = ttk.Label(frame, text=self._HELP_TEXT, wraplength=500)
        text.pack(fill=tk.X, padx=5, pady=(5, 0))

        mapping_list = ttk.Treeview(
            container,
            columns=("function", "key"),
            show=ttk.HEADINGS,
            height=5,
            padding=5,
        )
        for col in mapping_list["columns"]:
            mapping_list.heading(col, text=col.title(), anchor=tk.W)
        mapping_list.pack(fill=tk.X, padx=5, pady=(0, 5))

        mappings = 0
        for button in BUTTONS:
            if not button.keys:
                continue
            keys_string = ", ".join(map(self._get_display_key_name, button.keys))
            mapping_list.insert("", tk.END, mappings, values=(button.label, keys_string))
            mappings += 1

    def _get_display_key_name(self, key: str) -> str:
        return self._KEY_DISPLAY_MAP.get(key, key)

    def create_buttonbox(self, container: ttk.Frame) -> None:
        frame = ttk.Frame(container, padding=(5, 10))
        frame.pack(side=tk.BOTTOM, fill=tk.X, anchor=tk.S)

        submit = ttk.Button(
            master=frame,
            bootstyle="primary",
            text="OK",
            command=lambda: self._toplevel.destroy(),
        )
        submit.pack(padx=5, side=tk.RIGHT)
        submit.lower()  # set focus traversal left-to-right

        ttk.Separator(self._toplevel).pack(fill=tk.X)


class Remote(ttk.Frame):  # type: ignore
    def __init__(self, container: ttk.Frame, remote: Optional[PhilipsTVRemote] = None) -> None:
        super().__init__(container)

        self.remote = remote

        self._init_widgets()

    def _init_widgets(self) -> None:
        self.columnconfigure(0, weight=1)
        self.columnconfigure(1, weight=1)
        self.columnconfigure(2, weight=1)

        for row in set(map(lambda button: button.row, BUTTONS)):
            self.rowconfigure(row, weight=1)

        for button in BUTTONS:
            button.install(self, self._button_click, self._button_keypress)

        row += 1
        sep = ttk.Separator(self)
        sep.grid(row=row, column=0, columnspan=3, sticky=tk.NSEW, padx=1, pady=(1, 5))

        row += 1
        self._cap_toggle = ttk.Checkbutton(self, text="Capture keyboard", bootstyle="round-toggle")
        self._cap_toggle.grid(row=row, column=0, columnspan=3, sticky=tk.NSEW, padx=1, pady=(1, 5))

        cap_help = ttk.Button(
            self._cap_toggle, text="(help)", bootstyle="info-link", command=self._show_cap_help
        )
        cap_help.pack(side=tk.RIGHT)

        self.grid()

    def _button_click(self, key: InputKeyValue) -> None:
        self._send_key(key)

    def _button_keypress(self, key: InputKeyValue, *_: Any) -> None:
        if "selected" in self._cap_toggle.state():
            self._send_key(key)

    def _send_key(self, key: InputKeyValue) -> None:
        if not self.remote:
            not_paired_error(self)
        else:
            with handle_remote_errors(self):
                self.remote.input_key(key)

    def _show_cap_help(self) -> None:
        CapHelpDialog(self).show()



import tkinter as tk
from typing import Any, List, Optional

import ttkbootstrap as ttk
from philipstv.remote import PhilipsTVRemote

from philipstv_gui.errors import handle_remote_errors, not_paired_error


class Applications(ttk.Frame):  # type: ignore[misc]
    def __init__(self, container: ttk.Frame, remote: Optional[PhilipsTVRemote]) -> None:
        super().__init__(container)

        self.remote = remote
        self._apps_list: List[str] = []

        self._init_widgets()
        self.refresh()

    def _init_widgets(self) -> None:
        self.columnconfigure(0, weight=1)
        self.rowconfigure(0, weight=1)

        self._listbox_values = tk.StringVar()
        self._listbox = tk.Listbox(self, listvariable=self._listbox_values)
        self._listbox.grid(row=0, column=0, sticky=tk.NSEW)
        self._listbox.bind("<<ListboxSelect>>", self._on_selected)

        self.grid()

    def refresh(self) -> None:
        if not (self.remote and self.remote.auth):
            return
        with handle_remote_errors(self):
            self._apps_list = self.remote.get_applications()
            self._listbox_values.set(self._apps_list)  # type: ignore[arg-type]

    def _on_selected(self, _: Any) -> None:
        if not self.remote:
            not_paired_error(self)
        elif selection := self._listbox.curselection():  # type: ignore[no-untyped-call]
            selected_application = self._apps_list[selection[0]]
            with handle_remote_errors(self):
                self.remote.launch_application(selected_application)
